@@ -1,6 +1,6 @@
 /*
     This file is part of the KContacts framework.
-    Copyright (c) 2016 Laurent Montel <montel@kde.org>
+    Copyright (C) 2016-2017 Laurent Montel <montel@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@ class RoleTest : public QObject
 {
     Q_OBJECT
 public:
-    explicit RoleTest(QObject *parent = Q_NULLPTR);
+    explicit RoleTest(QObject *parent = nullptr);
     ~RoleTest();
 private Q_SLOTS:
     void shouldHaveDefaultValue();
